/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 *
 * @defgroup ble_sgds Stove Guard Installation Service Server
 * @{
 * @ingroup ble_sdk_srv
 *
 * @brief Stove Guard Installation Service Server module.
 *
 * @details This module implements ...
 *          During initialization, the module adds the LED Button Service and Characteristics
 *          to the BLE stack database.
 *
 *          The application must supply an event handler for receiving LED Button Service
 *          events. Using this handler, the service notifies the application when the
 *          LED value changes.
 *
 *          The service also provides a function for letting the application notify
 *          the state of the Button Characteristic to connected peers.
 *
 * @note The application must propagate BLE stack events to the LED Button Service
 *       module by calling ble_sgds_on_ble_evt() from the @ref softdevice_handler callback.
*/

#ifndef BLE_SGDS_H__
#define BLE_SGDS_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

#ifdef __cplusplus
extern "C" {
#endif

//	4EA1xxxx-7F31-41E8-82AE-6C88896E3797
	
#define SGDS_UUID_BASE        {0x97, 0x37, 0x6E, 0x89, 0x88, 0x6C, 0xAE, 0x82, 0xE8, 0x41, 0x31, 0x7F, 0x00, 0x00, 0xA1, 0x4E}

#define SGDS_UUID_SERVICE     			0x1920		// Base service
#define SGDS_UUID_SENSORPTR_CHAR		0x1921		// Sensor data pointer (Read + Write)
#define SGDS_UUID_SENSORMATRIX_CHAR 0x1922		// Sensor data matrix (Read)

//#define SGDS_UUID_COMMAND_CHAR		0x1923		// Sensor data pointer (Read + Write)

// Forward declaration of the ble_sgds_t type.
typedef struct ble_sgds_s ble_sgds_t;

typedef void (*ble_sgds_sptr_write_handler_t) (uint16_t conn_handle, ble_sgds_t * p_sgds, uint8_t new_state);


/** @brief LED Button Service init structure. This structure contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_sgds_sptr_write_handler_t sptr_write_handler; /**< Event handler to be called when the KEY Characteristic is written. */
} ble_sgds_init_t;

/**@brief LED Button Service structure. This structure contains various status information for the service. */
struct ble_sgds_s
{
    uint16_t                   				service_handle;      /**< Handle of Stove Guard HW Test Service (as provided by the BLE stack). */
    //ble_gatts_char_handles_t    			smatrix_char_handles;    /**< Handles related to the SEED Characteristic. */
    ble_gatts_char_handles_t    			sptr_char_handles; 		/**< Handles related to the KEY Characteristic. */
		//ble_gatts_char_handles_t    			command_char_handles; 
    uint8_t                     			uuid_type;           /**< UUID type for the StoveGuard Service. */
	  ble_sgds_sptr_write_handler_t 			sptr_write_handler;   /**< Event handler to be called when the LED Characteristic is written. */
};

/**@brief Function for initializing the LED Button Service.
 *
 * @param[out] p_sgds      LED Button Service structure. This structure must be supplied by
 *                        the application. It is initialized by this function and will later
 *                        be used to identify this particular service instance.
 * @param[in] p_sgds_init  Information needed to initialize the service.
 *
 * @retval NRF_SUCCESS If the service was initialized successfully. Otherwise, an error code is returned.
 */
uint32_t ble_sgds_init(ble_sgds_t * p_sgds, const ble_sgds_init_t * p_sgds_init);

/**@brief Function for handling the application's BLE stack events.
 *
 * @details This function handles all events from the BLE stack that are of interest to the LED Button Service.
 *
 * @param[in] p_sgds      LED Button Service structure.
 * @param[in] p_ble_evt  Event received from the BLE stack.
 */
void ble_sgds_on_ble_evt(ble_sgds_t * p_sgds, ble_evt_t * p_ble_evt);

/**@brief Function for sending a button state notification.
 *
 ' @param[in] conn_handle   Handle of the peripheral connection to which the button state notification will be sent.
 * @param[in] p_sgds         LED Button Service structure.
 * @param[in] button_state  New button state.
 *
 * @retval NRF_SUCCESS If the notification was sent successfully. Otherwise, an error code is returned.
 */

//uint32_t ble_sgds_on_tamper_change(uint16_t conn_handle, ble_sgds_t * p_sgds, uint8_t button_state);
uint32_t ble_sgds_on_sensorimage_change(uint16_t conn_handle, ble_sgds_t * p_sgds);

#ifdef __cplusplus
}
#endif

#endif // BLE_SGDS_H__

/** @} */
