/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 *
 * @defgroup ble_sghs LED Button Service Server
 * @{
 * @ingroup ble_sdk_srv
 *
 * @brief LED Button Service Server module.
 *
 * @details This module implements a custom LED Button Service with an LED and Button Characteristics.
 *          During initialization, the module adds the LED Button Service and Characteristics
 *          to the BLE stack database.
 *
 *          The application must supply an event handler for receiving LED Button Service
 *          events. Using this handler, the service notifies the application when the
 *          LED value changes.
 *
 *          The service also provides a function for letting the application notify
 *          the state of the Button Characteristic to connected peers.
 *
 * @note The application must propagate BLE stack events to the LED Button Service
 *       module by calling ble_sghs_on_ble_evt() from the @ref softdevice_handler callback.
*/

#ifndef BLE_SGHS_H__
#define BLE_SGHS_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

#ifdef __cplusplus
extern "C" {
#endif

//	D3B7xxxx-73D2-4F0F-A0B8-E61156461DF8
	
#define SGHS_UUID_BASE_NORDIC        {0x23, 0xD1, 0xBC, 0xEA, 0x5F, 0x78, 0x23, 0x15, \
                              0xDE, 0xEF, 0x12, 0x12, 0x00, 0x00, 0x00, 0x00}

#define SGHS_UUID_BASE        {0xF8, 0x1D, 0x46, 0x56, 0x11, 0xE6, 0xB8, 0xA0, 0x0F, 0x4F, 0xD2, 0x73, 0x00, 0x00, 0xB7, 0xD3}

#define SGHS_UUID_SERVICE     		0x1720		// Base service
#define SGHS_UUID_TAMPER_CHAR 		0x1721		// Tamper (Read + Notification)
#define SGHS_UUID_LED_CHAR    		0x1722		// LED (Read + Write)
#define SGHS_UUID_DIP_CHAR				0x1723		// DIP (Read + Notification)
#define SGHS_UUID_MKEY_CHAR	 		0x1724		// Magnet key (Read + Notification)
#define SGHS_UUID_SOUND_CHAR	 		0x1725		// Summer (Read + Write)
#define SGHS_UUID_BSTAT_CHAR	 		0x1726		// Breaker status (Read + Notification + Write)
#define SGHS_UUID_LSTAT_CHAR	 		0x1727		// Load status (Read + Notification + Write)
#define SGHS_UUID_VINT_CHAR	 		0x1728		// Internal Voltage (Read)
#define SGHS_UUID_VEXT_CHAR	 		0x1729		// External Voltage (Read)
#define SGHS_UUID_PRODTEST_CHAR	0x1730		// Production Test status  (Read + Notification + Write)
#define SGHS_UUID_CPUTEMP_CHAR	0x1735		// Die temperature  (Read + Notification)
#define SGHS_UUID_MBPOWER_CHAR	0x1731		// Multibreaker Power consumtion  (Read +Write)  //Kanske borde vara endast write
#define SGHS_UUID_STOVEGUARDSTATUS_CHAR	0x1732		// Multibreaker Power consumtion  (Read +Write)  //Kanske borde vara endast write




// Forward declaration of the ble_sghs_t type.
typedef struct ble_sghs_s ble_sghs_t;

typedef void (*ble_sghs_led_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t R_PWM, uint8_t G_PWM, uint8_t B_PWM, uint8_t blink_status);
typedef void (*ble_sghs_sound_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t frequency, uint8_t volume, uint8_t pattern);
typedef void (*ble_sghs_bstat_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t new_state);
typedef void (*ble_sghs_lstat_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t new_state);
typedef void (*ble_sghs_prodtest_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t new_state);
typedef void (*ble_sghs_mbpower_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t activePowerLowByte,uint8_t activePowerHighByte,uint8_t reactivePowerLowByte,uint8_t reactivePowerHighByte);
typedef void (*ble_sghs_stoveguardstatus_write_handler_t) (uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t statusID,uint8_t statusByte0,uint8_t statusByte1,uint8_t statusByte2,uint8_t statusByte3);


/** @brief LED Button Service init structure. This structure contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_sghs_led_write_handler_t led_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
    ble_sghs_sound_write_handler_t sound_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
    ble_sghs_bstat_write_handler_t bstat_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
    ble_sghs_lstat_write_handler_t lstat_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
    ble_sghs_prodtest_write_handler_t prodtest_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
		ble_sghs_mbpower_write_handler_t mbpower_write_handler; /**< Event handler to be called when the Multibreaker Characteristic is written. */
		ble_sghs_stoveguardstatus_write_handler_t stoveguardstatus_write_handler; /**< Event handler to be called when the StoveguardStatus Characteristic is written. */
} ble_sghs_init_t;

/**@brief LED Button Service structure. This structure contains various status information for the service. */
struct ble_sghs_s
{
    uint16_t                   				service_handle;      /**< Handle of Stove Guard HW Test Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t    			led_char_handles;    /**< Handles related to the LED Characteristic. */
    ble_gatts_char_handles_t    			tamper_char_handles; /**< Handles related to the TAMPER Characteristic. */
    ble_gatts_char_handles_t    			dip_char_handles; /**< Handles related to the DIP Characteristic. */
    ble_gatts_char_handles_t    			mkey_char_handles; /**< Handles related to the MAGNET KEY Characteristic. */
    ble_gatts_char_handles_t    			sound_char_handles; /**< Handles related to the SOUND Characteristic. */
    ble_gatts_char_handles_t    			bstat_char_handles; /**< Handles related to the BREAKER STATUS Characteristic. */
    ble_gatts_char_handles_t    			lstat_char_handles; /**< Handles related to the LOAD STATUS Characteristic. */
    ble_gatts_char_handles_t    			vint_char_handles; /**< Handles related to the INTERNAL VOLTAGE Characteristic. */
    ble_gatts_char_handles_t    			vext_char_handles; /**< Handles related to the EXTERNAL VOLTAGE Characteristic. */
    ble_gatts_char_handles_t    			prodtest_char_handles; /**< Handles related to the PRODUCTION TEST Characteristic. */
	  ble_gatts_char_handles_t    			mbpower_char_handles; /**< Handles related to the PRODUCTION TEST Characteristic. */
	  ble_gatts_char_handles_t    			stoveguardstatus_char_handles; /**< Handles related to the STOVEGUARD STATUS Characteristic. */
    ble_gatts_char_handles_t    			cputemp_char_handles; /**< Handles related to the CPU TEMPERATURE Characteristic. */
    uint8_t                     			uuid_type;           /**< UUID type for the StoveGuard Service. */
    ble_sghs_led_write_handler_t 			led_write_handler;   /**< Event handler to be called when the LED Characteristic is written. */
    ble_sghs_sound_write_handler_t 		sound_write_handler;   /**< Event handler to be called when the SUMMER Characteristic is written. */
    ble_sghs_bstat_write_handler_t 		bstat_write_handler;   /**< Event handler to be called when the BREAKER STATUS Characteristic is written. */
    ble_sghs_lstat_write_handler_t 		lstat_write_handler;   /**< Event handler to be called when the LOAD STATUS Characteristic is written. */
    ble_sghs_prodtest_write_handler_t 	prodtest_write_handler;   /**< Event handler to be called when the PRODUCTION TEST Characteristic is written. */
		ble_sghs_mbpower_write_handler_t 	mbpower_write_handler;   /**< Event handler to be called when the PRODUCTION TEST Characteristic is written. */
		ble_sghs_stoveguardstatus_write_handler_t stoveguardstatus_write_handler;
};

/**@brief Function for initializing the LED Button Service.
 *
 * @param[out] p_sghs      LED Button Service structure. This structure must be supplied by
 *                        the application. It is initialized by this function and will later
 *                        be used to identify this particular service instance.
 * @param[in] p_sghs_init  Information needed to initialize the service.
 *
 * @retval NRF_SUCCESS If the service was initialized successfully. Otherwise, an error code is returned.
 */
uint32_t ble_sghs_init(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init);

/**@brief Function for handling the application's BLE stack events.
 *
 * @details This function handles all events from the BLE stack that are of interest to the LED Button Service.
 *
 * @param[in] p_sghs      LED Button Service structure.
 * @param[in] p_ble_evt  Event received from the BLE stack.
 */
void ble_sghs_on_ble_evt(ble_sghs_t * p_sghs, ble_evt_t * p_ble_evt);

/**@brief Function for sending a button state notification.
 *
 ' @param[in] conn_handle   Handle of the peripheral connection to which the button state notification will be sent.
 * @param[in] p_sghs         LED Button Service structure.
 * @param[in] button_state  New button state.
 *
 * @retval NRF_SUCCESS If the notification was sent successfully. Otherwise, an error code is returned.
 */
uint32_t ble_sghs_on_tamper_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t button_state);

uint32_t ble_sghs_on_dip_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t dip_state);

uint32_t ble_sghs_on_mkey_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t mkey_state);

uint32_t ble_sghs_on_cputemp_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t temperature);

uint32_t ble_sghs_on_bstat_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t breaker_state);

uint32_t ble_sghs_on_stoveguardstatus_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t main_state);

uint32_t ble_sghs_vint_update(uint16_t conn_handle, ble_sghs_t * p_sghs, float voltage);

uint32_t ble_sghs_vext_update(uint16_t conn_handle, ble_sghs_t * p_sghs, float voltage);

#ifdef __cplusplus
}
#endif

#endif // BLE_SGHS_H__

/** @} */
