/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 *
 * @defgroup ble_sgis Stove Guard Installation Service Server
 * @{
 * @ingroup ble_sdk_srv
 *
 * @brief Stove Guard Installation Service Server module.
 *
 * @details This module implements ...
 *          During initialization, the module adds the LED Button Service and Characteristics
 *          to the BLE stack database.
 *
 *          The application must supply an event handler for receiving LED Button Service
 *          events. Using this handler, the service notifies the application when the
 *          LED value changes.
 *
 *          The service also provides a function for letting the application notify
 *          the state of the Button Characteristic to connected peers.
 *
 * @note The application must propagate BLE stack events to the LED Button Service
 *       module by calling ble_sgis_on_ble_evt() from the @ref softdevice_handler callback.
*/

#ifndef BLE_SGIS_H__
#define BLE_SGIS_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

#ifdef __cplusplus
extern "C" {
#endif

//	057Bxxxx-722D-4F83-A7E8-6F1C04FD9FEB
	
#define SGIS_UUID_BASE        {0xEB, 0x9F, 0xFD, 0x04, 0x1C, 0x6F, 0xE8, 0xA7, 0x83, 0x4F, 0x2D, 0x72, 0x00, 0x00, 0x7B, 0x05}

#define SGIS_UUID_SERVICE     		0x1820		// Base service
#define SGIS_UUID_COMMAND_CHAR		0x1821		// Command (Read + Write)
#define SGIS_UUID_RESULT_CHAR 		0x1822		// Result (Read)


// Forward declaration of the ble_sgis_t type.
typedef struct ble_sgis_s ble_sgis_t;

typedef void (*ble_sgis_command_write_handler_t) (uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t new_state);


/** @brief LED Button Service init structure. This structure contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_sgis_command_write_handler_t command_write_handler; /**< Event handler to be called when the COMMAND Characteristic is written. */
} ble_sgis_init_t;

/**@brief LED Button Service structure. This structure contains various status information for the service. */
struct ble_sgis_s
{
    uint16_t                   				service_handle;      /**< Handle of Stove Guard HW Test Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t    			command_char_handles;    /**< Handles related to the LED Characteristic. */
    ble_gatts_char_handles_t    			result_char_handles; /**< Handles related to the TAMPER Characteristic. */
    uint8_t                     			uuid_type;           /**< UUID type for the StoveGuard Service. */
    ble_sgis_command_write_handler_t command_write_handler;   /**< Event handler to be called when the LED Characteristic is written. */
};

/**@brief Function for initializing the LED Button Service.
 *
 * @param[out] p_sgis      LED Button Service structure. This structure must be supplied by
 *                        the application. It is initialized by this function and will later
 *                        be used to identify this particular service instance.
 * @param[in] p_sgis_init  Information needed to initialize the service.
 *
 * @retval NRF_SUCCESS If the service was initialized successfully. Otherwise, an error code is returned.
 */
uint32_t ble_sgis_init(ble_sgis_t * p_sgis, const ble_sgis_init_t * p_sgis_init);

/**@brief Function for handling the application's BLE stack events.
 *
 * @details This function handles all events from the BLE stack that are of interest to the LED Button Service.
 *
 * @param[in] p_sgis      LED Button Service structure.
 * @param[in] p_ble_evt  Event received from the BLE stack.
 */
void ble_sgis_on_ble_evt(ble_sgis_t * p_sgis, ble_evt_t * p_ble_evt);

/**@brief Function for sending a button state notification.
 *
 ' @param[in] conn_handle   Handle of the peripheral connection to which the button state notification will be sent.
 * @param[in] p_sgis         LED Button Service structure.
 * @param[in] button_state  New button state.
 *
 * @retval NRF_SUCCESS If the notification was sent successfully. Otherwise, an error code is returned.
 */

//uint32_t ble_sgis_on_tamper_change(uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t button_state);

uint32_t ble_sgis_result_update(uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t result);


#ifdef __cplusplus
}
#endif

#endif // BLE_SGIS_H__

/** @} */
