/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 * @defgroup nrf_twi_master_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Example Application main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 * @image html example_board_setup_a.jpg "Use board setup A for this example."
 */

#include "sdk_common.h"
#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "EEPROM.h"
#include "globals.h"


#define NRF_LOG_MODULE_NAME "EEPROM"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

// Buffer for data read from sensors.
#define BUFFER_SIZE  11
static uint8_t m_buffer[BUFFER_SIZE];

/* Indicates if operation on TWI has ended. */
static volatile bool EEPROM_xfer_done = false;

#if (BUFFER_SIZE < EEPROM_NUMBER_OF_REGISTERS)
    #error Buffer too small.
#endif

// Common callback function for all EEPROM functions
static void read_EEPROM_registers_cb(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("read_EEPROM_registers_cb - error: %d\r\n", (int)result);
        return;
    }

//    NRF_LOG_INFO("EEPROM:\r\n");
//    NRF_LOG_HEXDUMP_INFO(m_buffer, EEPROM_NUMBER_OF_REGISTERS);
		
		EEPROM_xfer_done = true;
		
}

// Get current acceleration values
uint8_t EEPROM_reg_addr = 0;

void EEPROM_GetData()
{
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        EEPROM_READ(&EEPROM_reg_addr, m_buffer, 6)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_EEPROM_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		EEPROM_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (EEPROM_xfer_done == false);
		
		uint8_t d1 = m_buffer[0];
		uint8_t d2 = m_buffer[1];
		uint8_t d3 = m_buffer[2];
		uint8_t d4 = m_buffer[3];
		uint8_t d5 = m_buffer[4];
		uint8_t d6 = m_buffer[5];
		
		NRF_LOG_INFO("Data: 1=%d, 2=%d, 3=%d, 4=%d, 5=%d, 6=%d\r\n", d1,d2,d3,d4,d5,d6);	
}		

// Set init configuration registers
static uint8_t init_config[] = { EEPROM_REG_CTRL_REG1, 0x01 };

app_twi_transfer_t EEPROM_init_device[EEPROM_INIT_TRANSFER_COUNT] =
{
    APP_TWI_WRITE(EEPROM_ADDR, init_config, sizeof(init_config), 0)
};


// Verify device is on bus
static uint8_t EEPROM_who_am_i = EEPROM_REG_WHO_AM_I;

bool EEPROM_verify_device()
{
	  static app_twi_transfer_t const transfers[] =
    {
        EEPROM_READ(&EEPROM_who_am_i, m_buffer, 1)
    };
		
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_EEPROM_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		EEPROM_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (EEPROM_xfer_done == false);

		if(m_buffer[0] == 0x4A)
			return true;
		else
			return false;
}
